//Boston University - Electronics Design Facility
//Monkey Timer Mk2
//http://edf.bu.edu/
//written by Chris Lawlor, January - April 2011
//Device is Atmega2560
//CPU clock is 16Mhz
//This file defines the beam control functions


#ifndef _MONKEY_MK2_BEAM_CONTROL_H
#define _MONKEY_MK2_BEAM_CONTROL_H



//RIGHT CHANNEL
//
#define RIGHT_LED0_BP		0	//PORTL0
#define RIGHT_LED1_BP 		1	//PORTL1
#define RIGHT_LED2_BP 		2	//PORTL2
#define RIGHT_LED3_BP 		3	//PORTL3
#define RIGHT_SENSOR0_BP	4	//PORTL4
#define RIGHT_SENSOR1_BP	5	//PORTL5
#define RIGHT_SENSOR2_BP	5	//PORTH5
#define RIGHT_SENSOR3_BP	6	//PORTH6
#define RIGHT_EXTRA_BP		4	//PORTB4

#define RIGHT_LED0_MSK		(1<<RIGHT_LED0_BP)		//PORTL0
#define RIGHT_LED1_MSK 		(1<<RIGHT_LED1_BP)		//PORTL1
#define RIGHT_LED2_MSK 		(1<<RIGHT_LED2_BP)		//PORTL2
#define RIGHT_LED3_MSK 		(1<<RIGHT_LED3_BP)		//PORTL3
#define RIGHT_SENSOR0_MSK	(1<<RIGHT_SENSOR0_BP)	//PORTL4
#define RIGHT_SENSOR1_MSK	(1<<RIGHT_SENSOR1_BP)	//PORTL5
#define RIGHT_SENSOR2_MSK	(1<<RIGHT_SENSOR2_BP)	//PORTH5
#define RIGHT_SENSOR3_MSK	(1<<RIGHT_SENSOR3_BP)	//PORTH6
#define RIGHT_EXTRA_MSK		(1<<RIGHT_EXTRA_BP)		//PORTB4

#define RIGHT_LED			(PINL)
#define RIGHT_LED0 			(PORTL 	& RIGHT_LED0_MSK)
#define RIGHT_LED1 			(PORTL	& RIGHT_LED1_MSK)
#define RIGHT_LED2 			(PORTL	& RIGHT_LED2_MSK)
#define RIGHT_LED3 			(PORTL 	& RIGHT_LED3_MSK)
#define RIGHT_SENSOR0		(PINL 	& RIGHT_SENSOR0_MSK)
#define RIGHT_SENSOR1		(PINL 	& RIGHT_SENSOR1_MSK)
#define RIGHT_SENSOR2		(PINH 	& RIGHT_SENSOR2_MSK)
#define RIGHT_SENSOR3		(PINH 	& RIGHT_SENSOR3_MSK)
#define RIGHT_EXTRA			(PINB 	& RIGHT_EXTRA_MSK)



//LEFT CHANNEL
//
#define LEFT_LED0_BP		5	//PORTB5
#define LEFT_LED1_BP 		6	//PORTB6
#define LEFT_LED2_BP 		7	//PORTB7
#define LEFT_LED3_BP 		4	//PORTE4
#define LEFT_SENSOR0_BP		5	//PORTE5
#define LEFT_SENSOR1_BP		5	//PORTG5
#define LEFT_SENSOR2_BP		3	//PORTE3
#define LEFT_SENSOR3_BP		3	//PORTH3
#define LEFT_EXTRA_BP		4	//PORTH4

#define LEFT_LED0_MSK		(1<<LEFT_LED0_BP)		//PORTB5
#define LEFT_LED1_MSK 		(1<<LEFT_LED1_BP)		//PORTB6
#define LEFT_LED2_MSK 		(1<<LEFT_LED2_BP)		//PORTB7
#define LEFT_LED3_MSK 		(1<<LEFT_LED3_BP)		//PORTE4
#define LEFT_SENSOR0_MSK	(1<<LEFT_SENSOR0_BP)	//PORTE5
#define LEFT_SENSOR1_MSK	(1<<LEFT_SENSOR1_BP)	//PORTG5
#define LEFT_SENSOR2_MSK	(1<<LEFT_SENSOR2_BP)	//PORTE3
#define LEFT_SENSOR3_MSK	(1<<LEFT_SENSOR3_BP)	//PORTH3
#define LEFT_EXTRA_MSK		(1<<LEFT_EXTRA_BP)		//PORTH4

#define LEFT_LED			(PINB)
#define LEFT_LED0 			(PORTB 	& LEFT_LED0_MSK)
#define LEFT_LED1 			(PORTB	& LEFT_LED1_MSK)
#define LEFT_LED2 			(PORTB	& LEFT_LED2_MSK)
#define LEFT_LED3 			(PORTE 	& LEFT_LED3_MSK)
#define LEFT_SENSOR0		(PINE 	& LEFT_SENSOR0_MSK)
#define LEFT_SENSOR1		(PING 	& LEFT_SENSOR1_MSK)
#define LEFT_SENSOR2		(PINE 	& LEFT_SENSOR2_MSK)
#define LEFT_SENSOR3		(PINH 	& LEFT_SENSOR3_MSK)
#define LEFT_EXTRA			(PINH 	& LEFT_EXTRA_MSK)



#define LEFT_CHANNEL	0x01
#define RIGHT_CHANNEL	0x02



volatile uint8_t active_channel = 0;
#define BEAM_IS_ACTIVE	active_channel

typedef uint8_t pulse_pattern_t;

//	 bit pattern for sensors and lights
//	
//	 S3 | S2 | S1 | S0 | L3 | L2 | L1 | L0
//
//	 1 for enabled, 0 disables
// 
//	 (bits S3 and L3 are ignored as there is no corresponding hardware)
//

#define PM_SENSOR3_BP 7
#define PM_SENSOR2_BP 6
#define PM_SENSOR1_BP 5
#define PM_SENSOR0_BP 4


#define SENSOR_PATERN_MSK	0x70	//SENSOR3 Left and Right are disabled
#define LED_PATERN_MSK		0x07	//LED3 Left and Right are disabled


#define SENSOR_READOUT_PERIOD			2660	//1995 for 8kHz	//798 20kHz for sample readouts; (was 3990 --> 4kHz); 1kHz frequency for analysis of the samples, sensor lock on time is ~200us
#define SENSOR_SAMPLE_LOCKOUT_PERIOD	5		//(3 for 3990)(19 for 798) 1ms divided down from SENSOR_READOUT_PERIOD
#define LED_PULSE_PERIOD				209		//76kHz frequency, actually it should be 210 or 211 with a 16Mhz clock, but 209 gives batter results.
#define PMCHANGE_LOCKOUT_PERIOD	 		5		//just delay 5ms, to allow the array to repopulate
#define PULSE_SWITCHING_PERIOD 			5		//5 for 83.33Hz (12ms); 9 for 50Hz (20ms); 24 for 20Hz operation, 50ms between pulse sequence switches
#define BEAM_INIT_TIME					36	 	//milliseconds, need to allow enough time for the all the pulse_sequence to cycle at least once; THIS IS ALSO THE RESOLUTION OF THE MEASUREMENTS
												//IE, ALL MEASUREMENTS ARE ACCURATE TO +/- BEAM_INIT_TIME

#define PULSE_SEQ_LENGTH 3

//L0 illuminates S0 and S1
//L1 illuminates S0, S1, and S2
//L2 illuminates S1 and S2
//hence the pattern sequence {0x31, 0x72, 0x64}

pulse_pattern_t pulse_sequence[PULSE_SEQ_LENGTH] = {0x11, 0x22, 0x44};//{0x77, 0x77, 0x77};

//#define TEST					//this setting ignores beam trips for sensor measurements w/ scope
//#define KILL_LEDS				//this setting shuts off normal LED toggling, so that LED lines can be toggled elsewhere to allow timing measurements w/ scope
//#define SENSOR_DIAGNOSTICS		//this setting turns on recording and USART transmission of the Sensor History at "trigger" points only, or continuously with CONTINUOUS also defined
//#define CONTINUOUS			//this setting specifies sensor diagnostics to happen throughout the sensor analysis process, otherwise its only when RecordSensorDiagnostics() is called
//#define LEDS_ACTIVE_LOW		//this setting optimizes for active low LEDs. it still works both ways, but it's more power efficient to shut them off when they're not used instead of on


volatile uint8_t pulse_sequence_index = 0;

volatile uint8_t S0_is_blocked;
volatile uint8_t S1_is_blocked;
volatile uint8_t S2_is_blocked;
volatile uint8_t beam_is_blocked;

volatile uint8_t pmchanged = 1;
volatile uint16_t pmchange_lockout = 0;
volatile uint32_t toggle_counter = 0;
volatile uint8_t pulse_switching_lockout = PULSE_SWITCHING_PERIOD;

#define BEAM_BLOCKED	(beam_is_blocked)
#define BEAM_OPEN		(!(BEAM_BLOCKED))


uint32_t elapsed_time_msec( uint32_t t2, uint32_t t1)
//returns elapsed time in milliseconds
{
	return ((uint32_t)(1000*((double)(t2-t1))/(2*38000)));
}


#define	ONTIME		76
#define OFFTIME		76
volatile uint8_t duty_cycle_lockout = ONTIME;	//led pulse train duty cycle adjustment
volatile uint8_t pulse_on = 1;				




void toggle_beam_leds()
{

	if (duty_cycle_lockout > 0)
	{
		duty_cycle_lockout--;
	}
	else
	{
		//switch the duty cycle
		if (pulse_on)
		{
			pulse_on = 0;
			duty_cycle_lockout = OFFTIME - 1;
		}
		else
		{
			pulse_on = 1;
			duty_cycle_lockout = ONTIME - 1;
			
			//
			//beginning of handle the pattern switching 
			//
			if (pulse_switching_lockout > 0)
			{
				pulse_switching_lockout--;
			}
			else
			{
				pulse_switching_lockout = PULSE_SWITCHING_PERIOD;
				pmchanged = 1;

				//increment the sequence
				pulse_sequence_index++;
				if (pulse_sequence_index == PULSE_SEQ_LENGTH)
				{
					pulse_sequence_index = 0;
				}
				pmchange_lockout = PMCHANGE_LOCKOUT_PERIOD;
			}
			//
			//end of handle the pattern switching
			//
		}
	}

	//if (pulse_on)
	//{
		uint8_t pulse_msk_led = pulse_sequence[pulse_sequence_index]&LED_PATERN_MSK;

		if (active_channel==LEFT_CHANNEL)
		{	
			//PINB |= (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
			//toggle leds according to the pulse pattern
			if (pulse_on)
			{
				if (pmchanged)
				{
					#ifndef KILL_LEDS

						//be careful for changes in the pulse patter
						#ifdef LEDS_ACTIVE_LOW
							PORTB = ( (PINB|(LED_PATERN_MSK<<5))&(~(pulse_msk_led<<5)));
						#else
							PORTB = ( (PINB&(~(LED_PATERN_MSK<<5)))| (pulse_msk_led<<5));				
						#endif

					#endif
					pmchanged = 0;
				}
				else
				{	
					#ifndef KILL_LEDS
						LEFT_LED |= (pulse_msk_led<<5);
					#endif
				}
			}
			else
			{
				#ifndef KILL_LEDS
					//turn LEDS off
					#ifdef LEDS_ACTIVE_LOW
						//make them high
						PORTB |= (((uint8_t)((LED_PATERN_MSK<<5))));
					#else
						//make them low
						PORTB = (((uint8_t)(~(LED_PATERN_MSK<<5))));
					#endif
				#endif
			}
		
		}
		else if (active_channel==RIGHT_CHANNEL)
		{
			//toggle leds according to the pulse pattern
			if (pulse_on)
			{
				if (pmchanged)
				{
					#ifndef KILL_LEDS
						//be careful for changes in the pulse patter
						PORTL = ((PINL&~LED_PATERN_MSK) | pulse_msk_led);	
					#endif			
					pmchanged = 0;
				}
				else
				{
					#ifndef KILL_LEDS
						RIGHT_LED |= pulse_msk_led;
					#endif
				}
			}
			else
			{
				#ifndef KILL_LEDS
					PORTL = ~LED_PATERN_MSK;
				#endif
			}
			
		}	
	//}
}






ISR(TIMER3_COMPA_vect, ISR_BLOCK)
//RUNS THE LED TOGGLING AT 76kHz (38kHz wave)
{
	//PINB |= (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
	toggle_counter++;
	toggle_beam_leds();
}


volatile uint8_t sensor_sample_lockout = SENSOR_SAMPLE_LOCKOUT_PERIOD;

//note the sample period is 1ms
#define NUM_SAMPLE_MS		4	//ie the number of 1ms sample periods to record
#define NUM_SAMPLES_PER_MS	6	//the number of samples per period (ms)
#define NUM_SENSOR_SAMPLES	NUM_SAMPLE_MS*NUM_SAMPLES_PER_MS	//total number of samples to store

uint8_t sensor_samples[NUM_SENSOR_SAMPLES];
uint8_t samples_index = 0;

inline void circ_inc(uint8_t *i)
{
	if (*i == NUM_SENSOR_SAMPLES - 1)
		*i = 0;
	else
		(*i)++;
}

inline void circ_dec(uint8_t *i)
{
	if (*i == 0)
		*i = NUM_SENSOR_SAMPLES - 1;
	else
		(*i)--;
}


#ifdef SENSOR_DIAGNOSTICS

void RecordSensorDiagnostics();
void SendSensorDiagnostics();

#define CCBC_MAX 100
uint8_t copy_count_before_change[CCBC_MAX][NUM_SAMPLE_MS+1];
uint8_t copy_samples[CCBC_MAX][NUM_SENSOR_SAMPLES];
uint8_t copy_pulse_msks[CCBC_MAX];
volatile uint8_t ccbc_index = 0;
volatile uint8_t test_me = 0;

#endif

uint8_t count_before_change[NUM_SAMPLE_MS+1];
uint8_t pulse_msk_sensor; 

inline uint8_t its_a_square_wave()
{

#ifdef SENSOR_DIAGNOSTICS
	//volatile uint8_t temp = test_me;
	//test_me++;	
#endif

	//PINB |= (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);

	uint8_t i, j, n; //k, l, m, n;
	uint8_t change = 0;
	//uint8_t count_before_change[NUM_SAMPLE_MS+1] = {0,0,0,0,0};
	memset(count_before_change, 0, NUM_SAMPLE_MS+1);	

	i = samples_index;
	circ_dec(&i);
	j = i;
	circ_dec(&j);
	n = 0;




	//USART_SendString(sensor_samples, NUM_SENSOR_SAMPLES);
	//USART_SendLineFeed();
	//USART_SendString(&pulse_msk_sensor, 1);
	//USART_SendString( (sensor_samples+i), 1);
	//USART_SendString( &samples_index, 1);
	//uint8_t temp = (sensor_samples[i]&pulse_msk_sensor);
	//USART_SendString( &temp, 1);
	//USART_SendLineFeed();

	
	while ( (n<NUM_SENSOR_SAMPLES) && (change<NUM_SAMPLE_MS+1))
	{
		count_before_change[change]++;		

		if (((sensor_samples[i]&pulse_msk_sensor)!=(sensor_samples[j]&pulse_msk_sensor)))
		{
			change++;
		}
		
		i = j;
		circ_dec(&j);
		n++;
	}
	
#ifdef SENSOR_DIAGNOSTICS
	#ifdef CONTINUOUS
		RecordSensorDiagnostics();
	#endif
#endif


#define LMX	13	//8   ????????????????????????????????????????????
#define LEQ	6
#define LMN	1	//4

	//if ((count_before_change[1]==LEQ)&&(count_before_change[2]==LEQ)&&(count_before_change[3]==LEQ)&&( (count_before_change[0]+count_before_change[4])==LEQ))
	if ((count_before_change[1]<LMX)&&(count_before_change[1]>LMN)&&(count_before_change[2]<LMX)&&(count_before_change[2]>LMN)&&(count_before_change[3]<LMX)&&(count_before_change[3]>LMN)&&( (count_before_change[0]+count_before_change[4])<LMX)&&((count_before_change[0]+count_before_change[4])>LMN))
	//if ( (count_before_change[1]<LMX)&&(count_before_change[2]<LMX)&&(count_before_change[3]<LMX)&&((count_before_change[0]+count_before_change[4])<LMX) )	
	{	
		//USART_SendString( &temp, 1);
		return 1;
	}
	else
	{
		//USART_SendString( &temp, 1);
		#ifdef TEST
			return 1;
		#endif
			
		return 0;
		
	}
}


void sensor_readout()
//SENSOR READOUT SECTION
{
	//PINB |= (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
	//PORTB = (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);

	//every 167us read out the sensors
	if (active_channel==LEFT_CHANNEL)
	{
		sensor_samples[samples_index] = ( ((LEFT_SENSOR3 >> LEFT_SENSOR3_BP)<<PM_SENSOR3_BP)|((LEFT_SENSOR2 >> LEFT_SENSOR2_BP)<<PM_SENSOR2_BP)|((LEFT_SENSOR1 >> LEFT_SENSOR1_BP)<<PM_SENSOR1_BP)|((LEFT_SENSOR0 >> LEFT_SENSOR0_BP)<<PM_SENSOR0_BP)); 	
	}
	else if (active_channel==RIGHT_CHANNEL)
	{
		sensor_samples[samples_index] = ( ((RIGHT_SENSOR3>>RIGHT_SENSOR3_BP)<<PM_SENSOR3_BP)|((RIGHT_SENSOR2>>RIGHT_SENSOR2_BP)<<PM_SENSOR2_BP)|((RIGHT_SENSOR1>>RIGHT_SENSOR1_BP)<<PM_SENSOR1_BP)|((RIGHT_SENSOR0>>RIGHT_SENSOR0_BP)<<PM_SENSOR0_BP)); 
	}	

	circ_inc(&samples_index);


	if (sensor_sample_lockout)
	{
		sensor_sample_lockout--;	
	}		
	else
	{
		//PINB |= (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
		//PORTB = (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);

		//every 1ms analyize the samples
		sensor_sample_lockout = SENSOR_SAMPLE_LOCKOUT_PERIOD;

		
		if (pmchange_lockout)
		{	
			//delay for enough time to repopulate the sensor_samples[] array (ie 4ms) plus the sensor lockon time of ~1ms, so 6ms should be safe
			pmchange_lockout--;
		}
		else
		{
			//check samples here
			//PORTB = (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
			//beam_is_blocked = !its_a_square_wave();

			uint8_t sreg_save = SREG;
			cli();
		    if (pmchange_lockout == 0)
			{
				pulse_msk_sensor = pulse_sequence[pulse_sequence_index]&SENSOR_PATERN_MSK; 
			}
			else
			{
				//the pulse_sequence_index was incrimented very recently and the 
				//go back and get the previous value for the test
				uint8_t temp = pulse_sequence_index;
				temp--;
				if (temp<0)
				{
					temp = PULSE_SEQ_LENGTH - 1;
				}
				pulse_msk_sensor = pulse_sequence[temp]&SENSOR_PATERN_MSK;
			}
			SREG = sreg_save;

			if (pulse_msk_sensor==(1<<PM_SENSOR0_BP))
			{
				S0_is_blocked = !its_a_square_wave();
			}
			else if (pulse_msk_sensor==(1<<PM_SENSOR1_BP))
			{
				S1_is_blocked = !its_a_square_wave();
			}
			else if (pulse_msk_sensor==(1<<PM_SENSOR2_BP))
			{
				S2_is_blocked = !its_a_square_wave();
			}
			else
			{
				//pulse_msk_sensor value not supported
				//only values 0x10, 0x20, or 0x40 
			}


			beam_is_blocked = (S0_is_blocked | S1_is_blocked | S2_is_blocked);


			//PORTB &= ~(LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
		}
		
		//PINB |= (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
		//PORTB &= ~(LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
	}

	//PINB |= (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
	//PORTB &= ~(LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
}


ISR(TIMER4_COMPA_vect, ISR_NOBLOCK)
//RUNS SENSOR READOUT AT 6kHz
{
	//PINB |= (LEFT_LED0_MSK|LEFT_LED1_MSK|LEFT_LED2_MSK);
	TIMSK4 = 0x00;	//disable this interrupt
	sensor_readout();
	TIMSK4 = 0x02;	//re-enable this interrupt
}



void setup_beam()
{
	//set DDR registers
	DDRL = ((DDRL&(~0x30))|0x0F);
	DDRH = ((DDRH&(~0x78))|0x00);
	DDRE = ((DDRE&(~0x38))|0x00);
	DDRG = ((DDRG&(~0x10))|0x00);
	DDRB = ((DDRB&(~0x00))|0xE0);

	PORTB |= 0x10;
	PORTH |= 0x10;

	pulse_sequence_index = 0;
	pulse_switching_lockout = PULSE_SWITCHING_PERIOD;
	pmchanged = 1;
	pmchange_lockout = PMCHANGE_LOCKOUT_PERIOD;
	duty_cycle_lockout = ONTIME;
	pulse_on = 1;
	sensor_sample_lockout = SENSOR_SAMPLE_LOCKOUT_PERIOD;

	memset(sensor_samples, 0, NUM_SENSOR_SAMPLES);
	samples_index = 0;

	S0_is_blocked = 1;
	S1_is_blocked = 1;
	S2_is_blocked = 1;
	beam_is_blocked = 1;


#ifdef SENSOR_DIAGNOSTICS
	memset(copy_count_before_change, 0, (NUM_SAMPLE_MS+1)*CCBC_MAX);
	memset(copy_samples, 0, CCBC_MAX*NUM_SENSOR_SAMPLES);
	memset(copy_pulse_msks, 0, CCBC_MAX);
	ccbc_index = 0;
	test_me = 0;
#endif

	//set up Timer3 for 76kHz operation for led toggling & pulse pattern switching
	TCCR3A = 0x00;
	TCCR3C = 0x00;
	OCR3A  = LED_PULSE_PERIOD;
	TIMSK3 = 0x02;
	TCCR3B = 0x09;

	//set up Timer4 for sensor readout at 6kHz
	TCCR4A = 0x00;
	TCCR4C = 0x00;
	OCR4A  = SENSOR_READOUT_PERIOD;
	TIMSK4 = 0x02;
	TCCR4B = 0x09;
}

void stop_beam()
{
	//turn off led toggeling timer and interrupt
	TCCR3B = 0x00;
	TIMSK3 = 0x00;

	//turn off sensor driver timer and interrupt
	TCCR4B = 0x00;
	TIMSK4 = 0x00;
	
	//turn all the leds off	
	PORTB = (PINB&~(LED_PATERN_MSK<<5));
	PORTL = (PINL&~LED_PATERN_MSK);

	//set DDR registers to inputs reduce power consumption?

	active_channel = 0;
}

#define ERRBEAMBLKD	-3
#define ERRNOCABLE	-2
#define ERRNOCHAN	-1


#define CABLE_UNPLUGGED		(((active_channel==LEFT_CHANNEL)&&(LEFT_EXTRA))||((active_channel==RIGHT_CHANNEL)&&(RIGHT_EXTRA)))


int8_t start_beam(uint8_t channel)
//returns > 0 on success, < 0 on error 
{	
	if ((channel!=LEFT_CHANNEL)&&(channel!=RIGHT_CHANNEL))
	{
		return ERRNOCHAN;
	}
	else
	{
		active_channel = channel;
		setup_beam();
		
		//check whether cable is plugged in
		if ( CABLE_UNPLUGGED )
		{
			stop_beam();
			return ERRNOCABLE;	
		}


		//test beam
		//wait to see if beam is functioning

		uint32_t t = toggle_counter;
		while (elapsed_time_msec(toggle_counter, t) < BEAM_INIT_TIME)
		{}

#ifdef KILL_LEDS
		while(1)
		{}
#endif

		if (BEAM_OPEN)
		{
			return 1;
		}
		else 
		{
			stop_beam();
			return ERRBEAMBLKD;
		}
	}	
}


#ifdef SENSOR_DIAGNOSTICS

void RecordSensorDiagnostics()
{
	//Record Sensor Diagnostics
	if (ccbc_index < CCBC_MAX)
	{
		copy_pulse_msks[ccbc_index] = pulse_msk_sensor;
		
		for (uint8_t z = 0; z<NUM_SAMPLE_MS+1; z++)
		{
			copy_count_before_change[ccbc_index][z] = count_before_change[z];
		}		

		uint8_t i = samples_index;

		for (uint8_t z = 0; z<NUM_SENSOR_SAMPLES; z++)
		{
			circ_dec(&i);
			copy_samples[ccbc_index][z] = sensor_samples[i];
		}

		ccbc_index++;
	}	
}


void SendSensorDiagnostics()
//Print out Sensor Diagnostics
{
	char sb[8];

	USART_SendString("Sensor Diagnostics\n",0);
	USART_SendString("Pulse Mask | Counts Before Change | Sensor State\n",0);
	for (uint8_t k = 0; k<ccbc_index; k++)
	{
		snprintf(sb, 8, "%02X ", copy_pulse_msks[k]);
		USART_SendString(sb, strlen(sb));
		USART_SendString("| ", 0);

		//USART_SendString((void*)(&(copy_count_before_change[k])), NUM_SAMPLE_MS+1);
		for(uint8_t i=0; i < (NUM_SAMPLE_MS+1); i++)
		{
			snprintf(sb, 8, "%2u ", copy_count_before_change[k][i]);
			USART_SendString(sb, strlen(sb));
		}
		//USART_SendLineFeed();
		USART_SendString("| ", 0);
		//USART_SendString((void*)(&(copy_samples[k])), NUM_SENSOR_SAMPLES);
		for(uint8_t i=0; i < NUM_SENSOR_SAMPLES; i++)
		{
			snprintf(sb, 8, "%02X ", copy_samples[k][i]);
			USART_SendString(sb, strlen(sb));
		}
		USART_SendLineFeed();
	}
	USART_SendString("End of Diagnostics\n",0);
}
#endif


#endif
