//Boston University - Electronics Design Facility
//Monkey Timer Mk2
//http://edf.bu.edu/
//written by Chris Lawlor, January - April 2011
//Device is Atmega2560
//CPU clock is 16Mhz
//This file contains the Menuing Framefork System definitions and all callback function definitions


#ifndef MONKEY_MK2_MENUS_H
#define MONKEY_MK2_MENUS_H


//The following are the Menuing Framefork System functions
//
// void populate_menu();
// void render_menus();
// void place_curser();
// void display_menu();
// void push_menu();
// void pop_menu();
// void up_keying_handler();
// void down_keying_handler();
// void select_keying_handler();
// void menu_button_handler();
// void run_menu_button_handler();



//The following are the Callback function declarations
//
result_t test_func();
result_t cancel_confirm();
result_t start_menu_clock();
result_t stop_menu_clock();
result_t set_date_time();
result_t start_left_channel();
result_t start_right_channel();
result_t terminal_mode();
result_t generate_trials_display();
result_t destroy_trials_display();
result_t main_menu_start();
result_t main_menu_stop();
result_t user_set_animal_number();
result_t display_animal_number();
result_t destroy_animal_number();
result_t generate_channel_display();
result_t destroy_channel_display();




//The following are other helper functions
//
// void 	display_menu_clock()
// void 	update_menu_clock()
// void 	find_menu_clock_line()
// void 	SendHelp()
// void 	NotYetImplemented()
// void 	SendIdentity()
// void 	menu_clock_digit_incriment()
// void 	test_menu_stack()
// uint8_t 	posmap()
// uint8_t 	beginning_of_same_date_field()
// uint8_t 	eep_fifo_usart_upload()
// uint8_t 	terminal_mode_user_confirm()
// result_t	terminal_mode_button_handler()
// result_t scroll_trials()
// result_t scroll_trials_up()
// result_t scroll_trials_down()



#include <string.h>
#include "monkey_mk2_display.h"
#include "monkey_mk2_frontpanel_buttons.h"
#include "monkey_mk2_menu_definitions.h"


#define MENU_STACK_SIZE		24			//way more than enough		
typedef uint8_t menu_index_t;			//uint8_t will only allow for a limit of 256 Menus, but that's probably enough for now
volatile menu_index_t	menu_stack[MENU_STACK_SIZE];
volatile menu_index_t	menu_stack_top = 0;


#define SCREENLINES	ROWS


typedef struct {							
	screen_t screen;								
    menu_index_t target[SCREENLINES];	//this gives each menu item another target menu 
	result_t (*func[SCREENLINES])();	//alternatively the target can be a function if FUNC is specified in target[]
	result_t (*on_load)();				//load event function pointer
	result_t (*on_unload)();			//unload event function
	uint8_t	flags;								
} menu_t;


static menu_t	rendered_menus[NUM_MENUS];		//watch out for RAM usage:   NUM_MENUS * sizeof(menu_t)
												//I was trying to avoid this but there's no way that I know of to 
												//render the menu from an integer variable at runtime with macros

typedef struct //active_menu
{
	menu_t menu;
	uint8_t selected_line;
	uint8_t	custom_content_flag;
	result_t result;
} active_menu_t;


active_menu_t active_menu;



//#define MENU_NUM(m)			MENU_##m##_NUM		
#define	MENU_DEFAULT_MSK	0x03		//default menu item flag mask
#define MENU_SELENL0_MSK	0x04		//line select enable flag masks
#define MENU_SELENL1_MSK	0x08
#define MENU_SELENL2_MSK	0x10
#define MENU_SELENL3_MSK	0x20

#define MAIN_MENU			MENU_0_NUM
#define LEFT_MENU			MENU_2_NUM
#define RIGHT_MENU			MENU_3_NUM
#define CONFIRM_MENU		MENU_10_NUM
#define TERMINAL_MODE_MENU	MENU_11_NUM
#define VIEW_TRIALS_MENU	MENU_5_NUM

//the macro MENU(n)	expands to the correct parameter list for populate_menu()
#define MENU(m)		MENU_##m##_NUM,MENU_##m##_LN0_TXT,MENU_##m##_LN1_TXT,MENU_##m##_LN2_TXT,MENU_##m##_LN3_TXT,MENU_##m##_LN0_TGT,MENU_##m##_LN1_TGT,MENU_##m##_LN2_TGT,MENU_##m##_LN3_TGT,MENU_##m##_LN0_FP,MENU_##m##_LN1_FP,MENU_##m##_LN2_FP,MENU_##m##_LN3_FP,MENU_##m##_FLAGS,MENU_##m##_ONLOAD,MENU_##m##_ONUNLOAD
	
void populate_menu(menu_index_t menu_number, char *l0, char *l1, char *l2, char *l3, menu_index_t t0, menu_index_t  t1, menu_index_t  t2, menu_index_t  t3, result_t (*fp0)(), result_t (*fp1)(), result_t (*fp2)(), result_t (*fp3)(),uint8_t menu_flags, result_t (*on_load)(), result_t (*on_unload)())
//the macro MENU(n)	expands to the correct parameter list for populate_menu()
//ie, you should just use the syntax:   populate_menu( MENU(k) );
{
	//set the screen 
	memset(&(rendered_menus[menu_number].screen), 0, sizeof(screen_t));
	display_str2scr( &(rendered_menus[menu_number].screen), l0, l1, l2, l3);

	//set the target array
	rendered_menus[menu_number].target[0] = t0;	
	rendered_menus[menu_number].target[1] = t1;
	rendered_menus[menu_number].target[2] = t2;
	rendered_menus[menu_number].target[3] = t3;

	rendered_menus[menu_number].func[0] = fp0;	
	rendered_menus[menu_number].func[1] = fp1;
	rendered_menus[menu_number].func[2] = fp2;
	rendered_menus[menu_number].func[3] = fp3;

	rendered_menus[menu_number].on_load = on_load;
	rendered_menus[menu_number].on_unload = on_unload;

	//set the flags
	rendered_menus[menu_number].flags = menu_flags;
}



void render_menus()
//this renders all the menus
{
	populate_menu( MENU(0) );
	populate_menu( MENU(1) );
	populate_menu( MENU(2) );
	populate_menu( MENU(3) );
	populate_menu( MENU(4) );
	populate_menu( MENU(5) );
	populate_menu( MENU(6) );
	populate_menu( MENU(7) );
	populate_menu( MENU(8) );
	populate_menu( MENU(9) );
	populate_menu( MENU(10) );
	populate_menu( MENU(11) );

	//did you remember to set NUM_MENUS correctly?

	memset(&active_menu, 0, sizeof(active_menu));
}



#define LINE_IS_SELECTABLE(j) ((active_menu.menu.target[(j)] != NOTGT)&&(active_menu.menu.target[(j)] != CLOCK))

#define CURSER	'>'


void place_curser(active_menu_t *amp)
{
	uint8_t	i = COLS - 1;
	
	if (!LINE_IS_SELECTABLE(amp->selected_line))
		return;

	for ( i = COLS - 1; i >= 0; i--)
	{
		if ( ( (amp->menu).screen[ (amp->selected_line) ][i] != ' ')  && ( (amp->menu).screen[ (amp->selected_line) ][i] != '\0') ) 
			break;
	}

	if (i<COLS-1)
	{	
		i++;	
	}

	(amp->menu).screen[amp->selected_line][i] = CURSER;

	//(amp->menu).screen[amp->selected_line][COLS-1] = CURSER;
}

static screen_t saved_screen;



void display_menu(menu_index_t menu_number, uint8_t active_selection)
//makes menu_number the active menu and displays it
//if active_selection is 0, then selected line is set to the default line
//if active_selection is 1, then selected line is not changed
//a bit set in active_menu.custom_content_flag will prevent refreshing the line to pre-rendered content, (ie if you don't want to clobber custom content)
{
	uint8_t ccf = active_menu.custom_content_flag;
	uint8_t j;

	if (ccf > 0)
	{
		for (j=0; j<SCREENLINES; j++)
		{
			if ((ccf>>j)&0x01)
				strncpy( saved_screen[j], active_menu.menu.screen[j], COLS);
		}
	}

	active_menu.menu = rendered_menus[menu_number];


	if (ccf > 0)
	{
		for (j=0; j<SCREENLINES; j++)
		{
			if ((ccf>>j)&0x01)
			{
				strncpy( active_menu.menu.screen[j], saved_screen[j], COLS);
			}
		}
	}
									
	if (active_selection == 0)
	{
		active_menu.selected_line = (rendered_menus[menu_number].flags&MENU_DEFAULT_MSK);
	}
	place_curser( &active_menu );

	//USART_SendScreen(&active_menu.menu.screen);

	//USART_SendString("display_menu():  calling display_screen...", 0);

	display_screen( &active_menu.menu.screen  );          //  <<<<<-----------------------------------------------------------------HERE!!!

	//USART_SendString("display_menu():  display_screen returned\n", 0);
}



void push_menu( menu_index_t menu_number)
{
	if (menu_stack_top<MENU_STACK_SIZE)
	{
		if (active_menu.menu.on_unload != NULL) 
		{
		//	USART_SendString("push_menu(): calling UNLOAD handler...", 0);
			active_menu.menu.on_unload();
		//	USART_SendString("UNLOAD handler returned\n", 0);
		}
		//else 
		//{
		//	USART_SendString("push_menu(): UNLOAD handler is NULL\n", 0);
		//}
	
		menu_stack[menu_stack_top] = menu_number;
		menu_stack_top++;
		display_menu(menu_number, 0);

		if (active_menu.menu.on_load != NULL)
		{
		//	USART_SendString("push_menu():  calling LOAD handler...", 0);
			active_menu.menu.on_load();
		//	USART_SendString("LOAD handler returned\n", 0);
		}
		//else 
		//{
		//	USART_SendString("push_menu(): LOAD handler is NULL\n", 0);
		//}
	}
}




void pop_menu()
{	
	if (menu_stack_top > 1)
	{
		if (active_menu.menu.on_unload != NULL) 
		{
		//	USART_SendString("pop_menu(): calling UNLOAD handler...", 0);
			active_menu.menu.on_unload();
		//	USART_SendString("UNLOAD handler returned\n", 0);
		}
		//else 
		//{
		//	USART_SendString("pop_menu(): UNLOAD handler is NULL\n", 0);
		//}

		menu_stack_top--;
		display_menu(menu_stack[menu_stack_top - 1], 0);

		if (active_menu.menu.on_load != NULL)
		{
		//	USART_SendString("pop_menu():  calling LOAD handler...", 0);
			active_menu.menu.on_load();
		//	USART_SendString("LOAD handler returned\n", 0);
		}
		//else 
		//{
		//	USART_SendString("pop_menu(): LOAD handler is NULL\n", 0);
		//}
	}
}



//----------------------------------------------------------
//----------------------------------------------------------

/*
void USART_SendStack()
{
	char m[MENU_STACK_SIZE+2];
	int i = 1;
	m[0] = 'S';
	
	for (i = 0; i<menu_stack_top; i++ )
	{
		m[i+1] = menu_stack[i] + 48;	//ascii conversion
	}
	
	m[i+1] = '\n';
	m[i+2] = 0;
	
	USART_SendString(m, 0);
}
*/

volatile uint8_t 	old_buttons = 0;


void up_keying_handler()
{
	//VALID UP KEYING
	//USART_SendString("UP\n", 0);
	int8_t j;
	if (active_menu.selected_line > 0) 
	{	
		for (j = active_menu.selected_line-1; j>-1; j--) 
		{
			if (LINE_IS_SELECTABLE(j))
			{
				break;
			}
		}
			if (j>-1)
		{
			active_menu.selected_line = j;
			display_menu(menu_stack[menu_stack_top-1], 1);
		}
	}
	//USART_SendStack();
}






void down_keying_handler()
{
	//VALID DOWN KEYING
	//USART_SendString("DOWN\n", 0);
	int8_t j;
	if (active_menu.selected_line < SCREENLINES-1) 
	{	
		for (j=active_menu.selected_line + 1; j<SCREENLINES; j++) 
		{
			if (LINE_IS_SELECTABLE(j))
			{
				break;
			}
		}
		if (j<SCREENLINES)
		{
			active_menu.selected_line = j;
			display_menu(menu_stack[menu_stack_top-1], 1);
		}
	}
	//USART_SendStack();	
}



void select_keying_handler()
{}



void menu_button_handler()
//interperets the frontpanel buttons in the context of the active_menu
//blocks execution until a new button is pressed
{

	uint8_t b = get_button_state();
	
	if (b == old_buttons)		//checking if they're old allows bypassing the blocking, in the case of an unhanded LMR event that happened during an previous handling subroutine
	{
		while ( !NEWBUTTON )
		{}

		b = get_button_state();
	}
	

	//always handle MENUBTN / LEFT / RIGHT first to have switch priority on this switch
	//on startup it will sense position if LEFT or RIGHT then push that menu state, if MENU ignore it
	
	if ( (b^old_buttons) & FP_BUTTON_LRM_MSK ) 
	{
		//handle LRM Change
		if (LEFT(b))
		{
			//USART_SendString("LEFT\n", 0);
			menu_stack_top = 1;
			push_menu(LEFT_MENU);
			//USART_SendStack();
		}
		else if	(RIGHT(b))
		{
			//USART_SendString("RIGHT\n", 0);
			menu_stack_top = 1;
			push_menu(RIGHT_MENU);	
			//USART_SendStack();	
		}
		else if (MENUBTN(b))
		{
			//USART_SendString("MENUBTN\n", 0);
			menu_stack_top = 0;
			push_menu(MAIN_MENU);
			//USART_SendStack();
		}
		else
		{}	//impossible to get here
	}
	else
	{
		//handle the pushbutton event
		if (VALID_UP(b))			
		{
			up_keying_handler();
		}	
		else if (VALID_DOWN(b))	
		{
			down_keying_handler();
		}	
		else if (VALID_SELECT(b))
		{
			//VALID SELECT KEYING
			//USART_SendString("SELECT\n", 0);

			menu_index_t tgt = active_menu.menu.target[active_menu.selected_line];

			if (tgt == BACKTGT)
			{
				pop_menu();
				//USART_SendString("POPPED MENU\n", 0);
			}
			else if ((tgt == NOTGT)||(tgt == CLOCK))
			{
				//do nothing
				//USART_SendString("ERROR -- NOTGT MENU\n", 0);
			}
			else if ((tgt>=0)&&(tgt<NUM_MENUS))
			{
				push_menu(tgt);
				//USART_SendString("PUSHED MENU\n", 0);
			}
			else if (tgt == FUNC)
			{
				active_menu.result = 0;
				if (active_menu.menu.func[active_menu.selected_line] != NULL)
				{	
					//char m[24];
					//USART_SendString("calling function...",0);
					active_menu.result = (*(active_menu.menu.func[active_menu.selected_line]))();
					//snprintf(m,24,"function returned %d\n", active_menu.result);
					//USART_SendString(m,0);		
				}
				else
				{
					//USART_SendString("function pointer is NULL",0);
				}
			}

			//USART_SendStack();
		}	
	}
	old_buttons = b;
}



void run_menu_button_handler()
{
	while(1)
	{
		menu_button_handler();
	}
}

/*
result_t test_func()
{
	USART_SendString("test_func():  THIS IS THE TEST FUNCTION\n", 0);
	return 44;
}
*/


result_t cancel_confirm()
{
	push_menu(CONFIRM_MENU);
	
	while(1)
	{
		uint8_t b = get_valid_pushbutton();
		
		if (VALID_UP(b))
		{
			up_keying_handler();
		}
		else if (VALID_DOWN(b))
		{
			down_keying_handler();
		}
		else if (VALID_SELECT(b))
		{
			if (active_menu.selected_line == 0)
			{
				//CANCEL
				pop_menu();
				return 0;
			}
			else if (active_menu.selected_line == 1)
			{
				//CONFIRM
				pop_menu();
				return 1;
			}
		}
	}	
}

a_time_t menu_clock;
uint8_t  clock_line;
uint8_t	 clock_acf_save;

void display_menu_clock()
{
	a_time_t_ToString( &menu_clock, active_menu.menu.screen[clock_line], COLS+1);
	//this relies on the returned string to be exactly COLS+1 including the '\0' char
	//add a line to accomicate the an early '\0' -- DONE!  this is now handled in display_screen()
	//snprintf(active_menu.menu.screen[clock_line], COLS+1, "Hello this is a test");
	//USART_SendString(active_menu.menu.screen[clock_line], 0);
	display_menu(menu_stack[menu_stack_top-1], 1);//redisplay active_menu
}


void update_menu_clock()
{
	a_time_t_Incriment(&menu_clock); 
	//ds1307_read_date_time(&menu_clock);
	display_menu_clock();
}


void find_menu_clock_line()
{
	for (clock_line = 0; clock_line<SCREENLINES; clock_line++)
	{
		if (active_menu.menu.target[clock_line] == CLOCK)
			break;
	}
}


result_t start_menu_clock()
{
	//char m[32];

	find_menu_clock_line();

	if (clock_line==SCREENLINES)
	{	
		//USART_SendString("NO CLOCK line\n",0);
		return 0;
	}
	else 
	{	
		clock_acf_save = active_menu.custom_content_flag;
		active_menu.custom_content_flag |= (1<<clock_line);
		//snprintf(m,32,"found CLOCK on line %d\n", clock_line);
		//USART_SendString(m,0);
		
	}
	
	ds1307_read_date_time(&menu_clock);
	display_menu_clock();

	//set up Timer counter interrupt to tick the clock display
	TCCR1A = 0x00;
	OCR1A  = 15625;
	TIMSK1 = 0x02;
	TCCR1B = 0x0D;

	//USART_SendString("start_menu_clock()...",0);
	return 1;
}



result_t stop_menu_clock()
{
	TIMSK1 = 0x00;
	TCCR1B = 0x00;
	//USART_SendString("stop_menu_clock()...",0);
	active_menu.custom_content_flag = clock_acf_save;

	return 1;
}

ISR(TIMER1_COMPA_vect, ISR_NOBLOCK)
{
	//USART_SendString("ISR(TIMER1_COMPA_vect)\n",0);
	update_menu_clock();
}


uint8_t mms_acf_save;


result_t main_menu_start()
{
	//read serial number and put it in line 1 (2nd line)
	mms_acf_save = active_menu.custom_content_flag;
	active_menu.custom_content_flag |= 0x02;
	snprintf((active_menu.menu.screen[1]), COLS+1, "%s.%s #%u", PRODUCT, VERSION, getDeviceSerialNumber());
	active_menu.menu.screen[1][COLS] = '\0'; //just in case!!!
	//snprintf(getDeviceSerialNumber()
	start_menu_clock();
	return 1;
}


result_t main_menu_stop()
{
	stop_menu_clock();
	active_menu.custom_content_flag = mms_acf_save;
	return 1;
}


/*
// Timestamp -- FOR REFERENCE
typedef struct a_time { 
	uint8_t month; 	// month 1-12
	uint8_t mday; 	// day 1-31
	uint8_t year; 	// year - 2000
	uint8_t hour; 	// hour 0-23
	uint8_t min; 	// minute 0-59
	uint8_t sec; 	// seconds 0-59
} a_time_t;
*/


uint8_t posmap(uint8_t j)
//"34 56  012  78 9A BC"
//"0123456789ABCDEF0123"
{
	switch(j)
	{
		case 0x00:
			return 0x07;
			break;
		case 0x01:
			return 0x08;
			break;
		case 0x02:
			return 0x09;
			break;
		case 0x03:
			return 0x00;
			break;
		case 0x04:
			return 0x01;
			break;
		case 0x05:
			return 0x03;
			break;
		case 0x06:
			return 0x04;
			break;
		case 0x07:
			return 0x0C;
			break;
		case 0x08:
			return 0x0D;
			break;
		case 0x09:
			return 0x0F;
			break;
		case 0x0A:
			return 0x10;
			break;
		case 0x0B:
			return 0x12;
			break;
		case 0x0C:
			return 0x13;
			break;
		default:
			return 0x00;	
	};
}

uint8_t beginning_of_same_date_field(uint8_t j)
//"34 56  012  78 9A BC"
//"0123456789ABCDEF0123"
{
	switch(j)
	{
		case 0x00:
		case 0x01:
		case 0x02:
			return 0x00;
			break;
		case 0x03:
		case 0x04:
			return 0x03;
			break;
		case 0x05:
		case 0x06:
			return 0x05;
			break;
		case 0x07:
		case 0x08:
			return 0x07;
			break;
		case 0x09:
		case 0x0A:
			return 0x09;
			break;
		case 0x0B:
		case 0x0C:
			return 0x0B;
			break;
		default:
			return 0x00;	
	};
}


void menu_clock_digit_incriment(uint8_t digit_num, uint8_t incriment)
//	incriment (or decriment) digit w/ rollover
//if incriment==1 circular inc of digit at position digit_num
//if incriment==0 circular dec of digit	at position digit_num
//digit_num is the particular digit in the date-time string
//circular rollover is not implemented
//"MM/DD/YYYY  HH:MM:SS"
//"34 56  012  78 9A BC"  <-- value of digit_num indicates position (cf. posmap() )
{
	//uint8_t digit;

	if (digit_num == 0)
	{
		//year hundreds 0-2 (because of uint8_t overflow)
		if (incriment)
		{
			if (menu_clock.year <= 155)
				menu_clock.year += 100;
		}
		else
		{
			if (menu_clock.year >= 100)
				menu_clock.year -= 100;
		}
	}
	else if (digit_num == 1)
	{
		//year tens
		if (incriment)
		{
			if (menu_clock.year <= 245)
			{
				if ((menu_clock.year/10)%10<9)
					menu_clock.year += 10;
			}
		}
		else
		{
			if (menu_clock.year >= 10)
			{
				if ((menu_clock.year/10)%10 > 0)
					menu_clock.year -= 10;
			}
		}
	}
	else if (digit_num == 2)
	{
		//year ones
		if (incriment)
		{
			if (menu_clock.year < 255)
			{	
				if ((menu_clock.year)%10 < 9)
					menu_clock.year += 1;
			}
		}
		else
		{
			if (menu_clock.year >= 1)
			{
				if ((menu_clock.year)%10 > 0)
					menu_clock.year -= 1;
			}
		}
	}
	else if (digit_num == 3)
	{
		//month tens 0-1
		if (incriment)
		{
			if (menu_clock.month/10 < 1)
			{	
				menu_clock.month += 10;
			}
		}
		else
		{
			if (menu_clock.month/10 > 0)
			{
				menu_clock.month -= 10;
			}
		}
	}
	else if (digit_num == 4)
	{
		//month ones
		//if tens == 0, 1-9
		//if tens == 1, 0-2
		if (incriment)
		{
			if (menu_clock.month < 12)
			{	
					menu_clock.month += 1;
			}
		}
		else
		{
			if (menu_clock.month > 1)
			{
					menu_clock.month -= 1;
			}
		}
	}
	else if (digit_num == 5)
	{
		//day tens
		uint8_t max = 3;

		if (menu_clock.month == 2)
			max = 2;

		if (incriment)
		{
			if (menu_clock.mday/10 < max)
			{	
				menu_clock.mday += 10;
			}
		}
		else
		{
			if (menu_clock.mday/10 > 0)
			{
				menu_clock.mday -= 10;
			}
		}
	}
	else if (digit_num == 6)
	{
		//day ones
		uint8_t max = 31;

		if ((menu_clock.month == 9)||(menu_clock.month == 4)||(menu_clock.month == 6)||(menu_clock.month == 11))
		{
			max = 30;
		}
		else if (menu_clock.month == 2)
		{
			if ( (menu_clock.year%4) == 0)
			{
				//leap year
				max = 29;
			}
			else 
			{
				max = 28;
			}
		}

		if (incriment)
		{
			if (menu_clock.mday < max)
			{	
				menu_clock.mday += 1;
			}
		}
		else
		{
			if (menu_clock.mday > 1)
			{
				menu_clock.mday -= 1;
			}
		}
	}
	else if (digit_num == 7)
	{
		//hour tens
		if (incriment)
		{
			if (menu_clock.hour/10 < 2)
			{	
				menu_clock.hour += 10;
			}
		}
		else
		{
			if (menu_clock.hour/10 > 0)
			{
				menu_clock.hour -= 10;
			}
		}
	}
	else if (digit_num == 8)
	{
		//hour ones
		if (incriment)
		{
			if (menu_clock.hour < 23)
			{	
				menu_clock.hour += 1;
			}
		}
		else
		{
			if (menu_clock.hour > 0)
			{
				menu_clock.hour -= 1;
			}
		}		
	}
	else if (digit_num == 9)
	{
		//min tens
		if (incriment)
		{
			if (menu_clock.min/10 < 5)
			{	
				menu_clock.min += 10;
			}
		}
		else
		{
			if (menu_clock.min/10 > 0)
			{
				menu_clock.min -= 10;
			}
		}
	}
	else if (digit_num == 10)
	{
		//min ones
		if (incriment)
		{
			if (menu_clock.min < 59)
			{	
				menu_clock.min += 1;
			}
		}
		else
		{
			if (menu_clock.min > 0)
			{
				menu_clock.min -= 1;
			}
		}
	}
	else if (digit_num == 11)
	{
		//sec tens
		if (incriment)
		{
			if (menu_clock.sec/10 < 5)
			{	
				menu_clock.sec += 10;
			}
		}
		else
		{
			if (menu_clock.sec/10 > 0)
			{
				menu_clock.sec -= 10;
			}
		}
	}
	else if (digit_num == 12)
	{
		//sec ones
		if (incriment)
		{
			if (menu_clock.sec < 59)
			{	
				menu_clock.sec += 1;
			}
		}
		else
		{
			if (menu_clock.sec > 0)
			{
				menu_clock.sec -= 1;
			}
		}
	}
}

volatile uint8_t set_date_acf_save;


result_t set_date_time()
{
	uint8_t done = 0;
	volatile uint8_t i;
	volatile uint8_t j;

	stop_menu_clock();
	set_date_acf_save = active_menu.custom_content_flag;
	active_menu.custom_content_flag = (1<<clock_line);
	
	do {

		for (i = 0; i<13; i++)
		{
			//remove menu curser from active menu (maybe)

			while(!done)
			{
				for (j = 0; j<posmap(i); j++)			//go to digit screen location	
				{
					display_curser_shift(0, 1, 0);  	//shift right by one
				}

				display_onoff(1, 1, 1, 0);				//blink screen curser

				uint8_t b = get_valid_pushbutton();		//wait for valid push buttons
		
				if (VALID_UP(b))
				{
					menu_clock_digit_incriment( i, 1);
				}
				else if (VALID_DOWN(b))
				{
					menu_clock_digit_incriment( i, 0);
				}
				else if (VALID_SELECT(b))
				{
					done = 1;							//go to next digit
				}

				if (!is_valid_date(&menu_clock))
				{
					//i = -1; //beginning_of_same_date_field(i);	//go back and do it again
					//ad an line saying invalid date
					active_menu.custom_content_flag |= (1<<(clock_line+1));

					volatile int8_t k = 0;
					if (clock_line < SCREENLINES - 1)
						k = 1;
					else 
						k = -1;

					if (k!=0)
						snprintf(active_menu.menu.screen[clock_line+k], COLS+1, "(invalid date/time) ");

				}
				else
				{
					//remove line for invalid date 
				
					active_menu.custom_content_flag &= ~(1<<(clock_line+1));
				}


				display_onoff(1, 0, 0, 0); 				//stop blinking screen curser
				display_home(0);
				display_menu_clock(); 					//redraw the screen
			}
			done = 0;
		}//for

	} while (!is_valid_date(&menu_clock));

	ds1307_set_date_time(&menu_clock);

	//maybe set selected line to BACK
	active_menu.selected_line = 3;
	//replace curser on active menu (not necessary here)
	active_menu.custom_content_flag = set_date_acf_save;
	start_menu_clock();
	return 1;
}




result_t run_trial(uint8_t channel)
{

#define READY_TIMEOUT_PERIOD  	30000
#define TRIAL_TIMEOUT_PERIOD 	30000

	volatile uint8_t b;
	volatile uint8_t trial_valid = 1;
	volatile uint8_t trial_canceled = 0;
	volatile uint8_t ready_canceled = 0;
	volatile uint8_t trial_timeout = 0;
	volatile uint8_t ready_timeout = 0;
	volatile uint8_t cable_fellout = 0;
	volatile uint8_t bb;
	volatile uint8_t bo;
	volatile  int8_t br;
	volatile uint32_t t0;
	volatile uint32_t start_time = 0;
	volatile uint32_t stop_time = 0;
	volatile uint8_t runtrial_acf_save;

	
	//SETUP PHASE
	//USART_SendString("\nrun_trial(): BEGIN\n", 0);
	//USART_SendString("custom content flag == ", 0);
	//USART_SendString(&active_menu.custom_content_flag, 1);
	//USART_SendLineFeed();

	//destroy_trials_display();
	destroy_channel_display();

	runtrial_acf_save = active_menu.custom_content_flag;
	active_menu.custom_content_flag = 0x0F;

	//USART_SendString("now, custom content flag == ", 0);
	//USART_SendString(&active_menu.custom_content_flag, 1);
	//USART_SendLineFeed();

	#ifdef SENSOR_DIAGNOSTICS
		USART_SendString("entered run_trial()\n", 0);
	#endif



	//check if eeprom fifo is full
	if (eep_fifo_is_full())
	{
		//display error message
		#ifdef SENSOR_DIAGNOSTICS
			USART_SendString("ERROR FIFO FULL\n", 0);
		#endif

		snprintf(active_menu.menu.screen[1], COLS+1, "Error Memory is Full");
		snprintf(active_menu.menu.screen[2], COLS+1, "Please Download Now");
		snprintf(active_menu.menu.screen[3], COLS+1, "OK");
		display_menu(menu_stack[menu_stack_top-1], 1);							//redisplay active_menu

		do {
			b=get_valid_pushbutton();
		} 
		while (!VALID_SELECT(b));
		
		active_menu.custom_content_flag = runtrial_acf_save;
		display_menu(menu_stack[menu_stack_top-1], 1); 
		//generate_trials_display();
		generate_channel_display();
		return 0;
	}	

	//get date time
	a_measure_t trial;	
	memset(&trial, 0, sizeof(a_measure_t));
	ds1307_read_date_time(&trial.time);

	//start the beam
	br = start_beam(channel);
	if ( br < 0 )
	{
		//Error beam not running
		//display error message
		#ifdef SENSOR_DIAGNOSTICS
			#ifndef CONTINUOUS
				RecordSensorDiagnostics();
			#endif
		#endif

		if (br == ERRBEAMBLKD)
		{
			#ifdef SENSOR_DIAGNOSTICS
				USART_SendString("ERROR NO BEAM\n", 0);
			#endif		
			snprintf(active_menu.menu.screen[1], COLS+1, "The Beam is Blocked");
			snprintf(active_menu.menu.screen[2], COLS+1, "PleaseRemoveBlockage"); 
		}
		else  //probably ERRNOCABLE, but could be ERRNOCHAN
		{
			#ifdef SENSOR_DIAGNOSTICS
				USART_SendString("ERROR NO CABLE\n", 0);
			#endif
			snprintf(active_menu.menu.screen[1], COLS+1, "Cable is Unplugged");
			snprintf(active_menu.menu.screen[2], COLS+1, "Please Check Cable"); 		
		}

		snprintf(active_menu.menu.screen[3], COLS+1, "OK");
		display_menu(menu_stack[menu_stack_top-1], 1);							//redisplay active_menu

		do {
			b=get_valid_pushbutton();
		} 
		while (!VALID_SELECT(b));
		
		active_menu.custom_content_flag = runtrial_acf_save;
		display_menu(menu_stack[menu_stack_top-1], 1); 
		//generate_trials_display();
		generate_channel_display();
		#ifdef SENSOR_DIAGNOSTICS
			SendSensorDiagnostics();
		#endif
		return 0;
	}
	
	#ifdef SENSOR_DIAGNOSTICS
		USART_SendString("Beam Started\n", 0);
	#endif

	//make a nice display
	snprintf(active_menu.menu.screen[1], COLS+1, "Beam Active");
	snprintf(active_menu.menu.screen[2], COLS+1, "00:00.00   (waiting)");  	//00.00 until started
	snprintf(active_menu.menu.screen[3], COLS+1, "CANCEL");
	display_menu(menu_stack[menu_stack_top-1], 1);							//redisplay active_menu

	//READY PHASE
	t0 = toggle_counter;

	//block until beam broken
	bo = BEAM_OPEN;
	cable_fellout = CABLE_UNPLUGGED;
	while(!ready_canceled && !ready_timeout && !cable_fellout && bo)
	{
		if (elapsed_time_msec(toggle_counter, t0) > READY_TIMEOUT_PERIOD )
		{
			ready_timeout = 1;
		}

		if (NEWBUTTON)
		{
			b = get_button_state();
			if (MENUBTN(b) || VALID_SELECT(b))
			{
				ready_canceled = 1;
			}	
		}
		bo = BEAM_OPEN;
		cable_fellout = CABLE_UNPLUGGED;
	}


	//TRIAL PHASE
	if (!ready_canceled && !ready_timeout && !cable_fellout)
	{
		//get starting  count
		start_time = toggle_counter;

		#ifdef SENSOR_DIAGNOSTICS
			#ifndef CONTINUOUS
				RecordSensorDiagnostics();
			#endif
		#endif

		//update the menu
		snprintf(active_menu.menu.screen[1], COLS+1, "Beam Triggered");
		snprintf(active_menu.menu.screen[2], COLS+1, "**:**.**    (timing)");
		snprintf(active_menu.menu.screen[3], COLS+1, "CANCEL");
		display_menu(menu_stack[menu_stack_top-1], 1);	
		bb = BEAM_BLOCKED;
		cable_fellout = CABLE_UNPLUGGED;
		while(!trial_canceled && !trial_timeout && !cable_fellout && bb)
		{

			//update display at 100Hz
			//ss.tht while running
			//ticking at hundredths
			//with a cancel option
		
			//NOTE:  writing the screen takes ~6ms and used a 25kHz Non-Blocking Interrupt		
			//which caused screen lockups due to starvation by competition with the 
			//76kHz beam timer and it's Blocking Interrupt
			//Consequently, I had to make the screen interrupt Blocking to resolve the lockups
			//but this will skew the 76kHz Beam Interrupt, which will yeild measurement errors
			//AND/OR beam dropout due to deviation from the sensor's 38kHz baseline frequency 
			//So, NO screen updating durring timing
			//unless a different scheme is hatched for writing the screen
			//(one possable way might be to block and poll the display ready bit, instead of using the timer & interrupt)
			//(or to block using a timer without an interrupt by just polling the interrupt flag)
			//(or a comnination of the two schemes)

			//handle the timeout
			if ( elapsed_time_msec(toggle_counter, start_time) > TRIAL_TIMEOUT_PERIOD )
			{
				trial_valid = 0;
				trial_timeout = 1;
			}


			//and handle user cancel action 
			//(select button or toggle switch into menu position)
			//(technically the other channel's menu switch should also be a cancel)
			//(but I can't physically move the switch fast enough to skip the MENUBTN position, so I think it's moot)
			if (NEWBUTTON)
			{
				b = get_button_state();
				if (MENUBTN(b) || VALID_SELECT(b))
				{
					trial_valid = 0;
					trial_canceled = 1;
				}	
			}

			bb = BEAM_BLOCKED;

			cable_fellout = CABLE_UNPLUGGED;
			

		}//while(BEAM_BLOCKED)
	}//if (!user_canceled && !ready_timeout)


	//END PHASE

	stop_time = toggle_counter;			//get ending count
	stop_beam();
	
	#ifdef SENSOR_DIAGNOSTICS
		#ifndef CONTINUOUS
			RecordSensorDiagnostics();
		#endif

		USART_SendString("Beam Stopped\n", 0);
	#endif
	



	//populate the a_measure_t structute
	if (ready_timeout || ready_canceled)
	{
		trial.interval = (uint16_t)(elapsed_time_msec(stop_time, t0)/10);
	}
	else
	{
		trial.interval = (uint16_t)(elapsed_time_msec(stop_time, start_time)/10);
	}

	if (channel==LEFT_CHANNEL)
		trial.flags |= LEFT_MASK;		//LEFT_MASK		0x01	//Left/Right -> bit0 1/0

//BEGIN RELOCATE SECTION
/*
	trial_valid = (!ready_canceled && !ready_timeout && !trial_canceled && !trial_timeout  && !cable_fellout);

	if (trial_valid)
		trial.flags |= VALID_MASK;		//VALID_MASK	0x02	//Valid/Invalid -> bit1 1/0
	
	trial.flags |= (get_animal_number()<<3);

	//if (!user_canceled && !timeout)
	//{
		//write it to the eeprom fifo
		eep_fifo_push_last(&trial);
		//or use
		//uint8_t r = eep_fifo_push_last(&trial);
		//if (r < 0)
		//{
		//	//failed to write to eeprom
		//	//display error message
		//	USART_SendString("EPROM FIFO WRITE FAILED\n", 0);
		//	active_menu.custom_content_flag = runtrial_acf_save;
		//	display_menu(menu_stack[menu_stack_top-1], 1);
		//	return 0;
		//}
	//}//if (!user_canceled && !timeout) #2
*/
//END RELOCATE SECTION
	
	snprintf(active_menu.menu.screen[1], COLS+1, "Beam Stopped");

	if (cable_fellout)
	{
		snprintf(active_menu.menu.screen[2], COLS+1, "Error Cable Fell Out");
		#ifdef SENSOR_DIAGNOSTICS
			USART_SendString("Error Cable Fell Out\n", 0);
		#endif
	}
	else if (ready_canceled || trial_canceled) 
	{
		snprintf(active_menu.menu.screen[2], COLS+1, "%02u:%02u.%02u  (canceled)", (trial.interval/100)/60, (trial.interval/100)%60, (trial.interval)%100);
		#ifdef SENSOR_DIAGNOSTICS
			USART_SendString("Canceled\n", 0);
		#endif
	}	
	else if (ready_timeout || trial_timeout)
	{
		snprintf(active_menu.menu.screen[2], COLS+1, "%02u:%02u.%02u   (timeout)", (trial.interval/100)/60, (trial.interval/100)%60, (trial.interval)%100);
		#ifdef SENSOR_DIAGNOSTICS
			USART_SendString("Timeout\n", 0);
		#endif
	}
	else
	{
		snprintf(active_menu.menu.screen[2], COLS+1, "%02u:%02u.%02u   (elapsed)", (trial.interval/100)/60, (trial.interval/100)%60, (trial.interval)%100);
	}

#define VALIDPROMPT 	"Inval=DN Val=SEL OK"
#define INVALIDPROMPT 	"Invalid  OK"

	trial_valid = (!ready_canceled && !ready_timeout && !trial_canceled && !trial_timeout  && !cable_fellout);

	if (trial_valid)
	{
		snprintf(active_menu.menu.screen[3], COLS+1, VALIDPROMPT);
	}
	else
	{
		snprintf(active_menu.menu.screen[3], COLS+1, INVALIDPROMPT);
	}

	display_menu(menu_stack[menu_stack_top-1], 1);	

	do {
		b=get_valid_pushbutton();
	} 
	while ( (!VALID_SELECT(b)) && (!VALID_DOWN(b)) );  //VALID_DOWN(b) indicates user_invalidated

	uint8_t user_invalidated = 0;;

	if (trial_valid)
		user_invalidated = VALID_DOWN(b);
	
//SET THE TRIAL FLAGS
//COMMIT TRIAL TO EEPROM

//BEGIN RELOCATE SECTION

	trial_valid &= !user_invalidated;

	if (trial_valid)
		trial.flags |= VALID_MASK;		//VALID_MASK	0x02	//Valid/Invalid -> bit1 1/0
	
	trial.flags |= (get_animal_number()<<3);

	//if (!user_canceled && !timeout)
	//{
		//write it to the eeprom fifo
		eep_fifo_push_last(&trial);
		//or use
		//uint8_t r = eep_fifo_push_last(&trial);
		//if (r < 0)
		//{
		//	//failed to write to eeprom
		//	//display error message
		//	USART_SendString("EPROM FIFO WRITE FAILED\n", 0);
		//	active_menu.custom_content_flag = runtrial_acf_save;
		//	display_menu(menu_stack[menu_stack_top-1], 1);
		//	return 0;
		//}
	//}//if (!user_canceled && !timeout) #2

//END RELOCATE SECTION


	active_menu.custom_content_flag = runtrial_acf_save;
	display_menu(menu_stack[menu_stack_top-1], 1); 
	//generate_trials_display();
	generate_channel_display();
	
	#ifdef SENSOR_DIAGNOSTICS
		USART_SendString("leaving run_trial()\n", 0);
		SendSensorDiagnostics();
	#endif

	return 1;
}



result_t start_left_channel()
{
	run_trial(LEFT_CHANNEL);
	return 1;
}

result_t start_right_channel()
{
	run_trial(RIGHT_CHANNEL);
	return 1;
}


uint8_t eep_fifo_usart_upload()
//uploads the fifo over the USART without erasing (popping) any data
//this allows to have data persistance during the "DOWNLOAD" command
//an additional call to eep_fifo_clear() would clear the entire fifo
{
	a_measure_t data;
#define SMAX	48
	char s[SMAX];
	if (eep_fifo_is_empty())
	{
		USART_SendString("FIFO EMPTY\n", 0);
		return 0;
	}
	else
	{
		uint8_t copy_SREG = SREG;
		cli();
		uint16_t last = eeprom_read_word(&eep_fifo_last);
		uint16_t first = eeprom_read_word(&eep_fifo_first);
		USART_SendString("Date,Time,AnimalNumber,Elapsed,Channel,Valid,AlreadyDownloaded\n", 0);
		while (first != last)
		{
			read_a_measure_t(&data, &eep_fifo[first]);
			memset(s, 0, SMAX);
			a_measure_t_ToString(&data, s, SMAX);
			USART_SendString(s, 0);
			data.flags |= IS_DOWNLOADED;
			write_a_measure_t(&data, &eep_fifo[first]);
			//eeprom_write_byte(&(eep_fifo[first].flags), data.flags);
			circular_inc(&first);
		}
		SREG = copy_SREG;
	}

	return 1;
}


void SendHelp()
//	Commands
//  ---------------
//	1. identify 
//	2. help
//	3. echo
//	4. start <L|R>
//	5. cancel
//	6. download
//	7. init
{
	USART_SendString("Valid commands are: ID, HELP, ECHO, START <L|R>, CANCEL, DOWNLOAD, RESET, SET, EXIT", 0);
}


void NotYetImplemented()
{
	USART_SendString("NOT YET IMPLEMENTED", 0);
}


void SendIdentity()
{
#define CBMAX 32
	char cb[CBMAX];
	USART_SendLineFeed();
	USART_SendString(BU, 0);
	USART_SendLineFeed();
	USART_SendString(EDF, 0);
	USART_SendLineFeed();
	snprintf(cb, CBMAX, "%s.%s SN#%u\n", PRODUCT, FWV, getDeviceSerialNumber());
	USART_SendString(cb, 0);
}


#define PROMPT 		"?> "


result_t terminal_mode_button_handler()
{
	if (NEWBUTTON)
	{	
		uint8_t b = get_button_state();
		if (VALID_SELECT(b))
			return 1;
	}
	return 0;
}


uint8_t terminal_mode_user_confirm(char *s, result_t (*breakout_handler)(), result_t *breakout)
{
	uint8_t yes = 0;
	uint8_t no = 0;
	do {
		USART_SendString(s, 0);
		USART_SendString("  [YES|NO]\n", 0);
		USART_ReceiveString(usart_buffer, USART_BUFFMAX, breakout_handler, breakout);
		if ((breakout!=NULL)&&(*breakout))
		{
			break;
		}
		yes = ( strncmp(usart_buffer, "YES", 3) == 0);
		no  = ( strncmp(usart_buffer,  "NO", 2) == 0);
	} while (!(yes||no));
	
	return yes;
}



result_t terminal_mode()
{
	push_menu(TERMINAL_MODE_MENU);

	result_t breakout = 0;
	
	SendIdentity();
	USART_SendString("type DOWNLOAD to download trials\n", 0);
	USART_SendString("type HELP for a list of commands", 0);	

	while(!breakout)
	{
		USART_SendLineFeed();
		USART_SendString(PROMPT, 0);
		ClearBuffer(usart_buffer, USART_BUFFMAX);
		USART_ReceiveString(usart_buffer, USART_BUFFMAX, terminal_mode_button_handler, &breakout);	
		USART_SendLineFeed(0);
		
		if (breakout)
		{
			//do nothing here and skip all the other commands
		}
		else if (strlen(usart_buffer) == 0)
		{
			//do nothing
		}
		else if (strncmp(usart_buffer, "ID", 2) == 0)
		{
			SendIdentity(0);	
		}
		else if (strncmp(usart_buffer, "HELP", 4) == 0)
		{
			SendHelp();
		}
		else if (strncmp(usart_buffer, "ECHO", 4) == 0)
		{

			USART_echotest();
		}
		else if (strncmp(usart_buffer, "START", 5) == 0)
		{
			NotYetImplemented();
		}
		else if (strncmp(usart_buffer, "CANCEL", 6) == 0)
		{
			NotYetImplemented();
		}
		else if (strncmp(usart_buffer, "DOWNLOAD", 8) == 0)
		{
			//NotYetImplemented();
			if (eep_fifo_usart_upload())
			{
				if (terminal_mode_user_confirm("Clear data now?", terminal_mode_button_handler, &breakout))
				{
					eep_fifo_clear();
					USART_SendString("Fifo cleared\n", 0);
				}
			}
		}
		
		else if (strncmp(usart_buffer, "SET", 3) == 0)
		{
			NotYetImplemented();
		}
		
		else if (strncmp(usart_buffer, "EXIT", 5) == 0)
		{
			//NotYetImplemented();
			USART_SendString("Terminal Session Closed\n", 0);
			break;
		}
	
		else if (strncmp(usart_buffer, "RESET", 4) == 0)
		{
			USART_SendString("WARNING - THIS WILL CLEAR ALL DATA!!!\n", 0);
			if (terminal_mode_user_confirm("Are you sure?", terminal_mode_button_handler, &breakout))
			{
				int t;
				if ((t=ds1307_init()) > 0)
				{
					eep_fifo_clear();
					USART_SendString("Reset succeeded\n",0);
				}
				else
				{
					char tb[16];
					snprintf(tb, 16, "Reset failed error code %d\n", t);
					USART_SendString(tb,0);
				}
			}
		}
		else 
		{
			USART_SendString("UNKNOWN COMMAND.  TRY 'HELP'", 0);
		}
	
		if (breakout)
		{
			USART_SendString("Terminal Mode Canceled by Remote User\n", 0);
			break;
		}

	}//while()

	pop_menu();
	return 1;
}


//#define LEFT_CHANNEL	0x01
//#define RIGHT_CHANNEL	0x02
uint8_t	scroll_channels	= 0;	//generates the filter
uint8_t scroll_channe_line = 0;
volatile uint8_t scroll_trails_acf_save;
volatile uint16_t local_fifo_index = 0;
//eep_fifo_peek(a_measure_t *data, uint16_t n)


result_t scroll_trials(uint8_t n)
//n == -1, scroll to newer trials
//n ==  0, simply generate the display from the newest data
//n ==  1, scroll to older trials
{
	a_measure_t temp;
	int8_t	r = 1;
	//uint8_t num_lines = 0;

	int8_t starting_line = 0;
	int8_t ending_line = 0;

	uint8_t channel = 0;
	local_fifo_index = 0;
	//figure out what channel(s) to display based on the menu stack
	if (menu_stack[menu_stack_top - 1] == LEFT_MENU)
	{
		//num_lines = 3;
		starting_line = 2;
		ending_line = 1;
		channel = LEFT_CHANNEL;
	}
	else if (menu_stack[menu_stack_top - 1]==RIGHT_MENU)
	{
		//num_lines = 3;
		starting_line = 2;
		ending_line = 1;
		channel = RIGHT_CHANNEL;
	}
	else if (menu_stack[menu_stack_top - 1]==VIEW_TRIALS_MENU)
	{
		//num_lines = 3;
		starting_line = 2;
		ending_line = 0;
		channel = (RIGHT_CHANNEL|LEFT_CHANNEL);
	}

	if (n==0)
	{
		for (uint8_t k = starting_line; ((k>=ending_line) && (r>0)); k--)
		{
			//scan the eep_fifo looking for trials for the channel(s)
			do{
				r = eep_fifo_peek_last(&temp, local_fifo_index);
				local_fifo_index++;
				//USART_SendString("H\n", 0);
			}while ( (r>0) && (((!(channel&LEFT_CHANNEL))&&(temp.flags&LEFT_MASK))||((!(channel&RIGHT_CHANNEL))&&(!(temp.flags&LEFT_MASK)))) );
			
			if (r>0)
			{
				//print to active menu	
				a_measure_t_ToDisplayString(&temp, active_menu.menu.screen[k], COLS+1);
			}
		}
	}
	else if (n == 1)
	{
		//copy active_menu.menu.screen[1] to active_menu.menu.screen[2]
		//loop:	
		//scan the eep_fifo looking for next older trial for the channel(s)	
		//print it to active menu
		//or print (none) if none older		
	}
	else if (n == -1);
	{
		//copy active_menu.menu.screen[2] to active_menu.menu.screen[1]
		//loop:	
		//scan the eep_fifo looking for next newer trial for the channel(s)	
		//print it to active menu
		//or print (none) if none older		
	}


	//redraw the active menu
	display_menu(menu_stack[menu_stack_top-1], 1);

	return 1;
}

result_t generate_trials_display()
{
	//set the custom content flag
	local_fifo_index = 0;
	scroll_trails_acf_save = active_menu.custom_content_flag;
	if ((menu_stack[menu_stack_top - 1]==LEFT_MENU) || (menu_stack[menu_stack_top - 1]==RIGHT_MENU))
	{
		active_menu.custom_content_flag |= 0x06;
	}
	else if (menu_stack[menu_stack_top - 1]==VIEW_TRIALS_MENU)
	{
		active_menu.custom_content_flag |= 0x07;
	}
	return scroll_trials(0);
}

result_t destroy_trials_display()
{
	//clear the custom content flag
	active_menu.custom_content_flag = scroll_trails_acf_save;
	return 1;
}

result_t scroll_trials_up()
{
	return scroll_trials(1);
}


result_t scroll_trials_down()
{
	return scroll_trials(-1);
}

//uint8_t gcd_acf_save;

result_t generate_channel_display()
{
	//gcd_acf_save = active_menu.custom_content_flag;
	generate_trials_display();
	active_menu.custom_content_flag |= 0x01;
	snprintf(active_menu.menu.screen[0]+COLS-4, 5, "A#%2u", get_animal_number());
	display_menu(menu_stack[menu_stack_top-1], 1);//redisplay active_menu
	return 1;
}

result_t destroy_channel_display()
{
	destroy_trials_display();
	//gcd_acf_save = active_menu.custom_content_flag;
	return 1;
}


uint8_t san_acf_save;

result_t user_set_animal_number()
{
	//read animal number
	uint8_t animal_num = get_animal_number();
	uint8_t b;	

	do {

		snprintf(active_menu.menu.screen[1], COLS+1, "%2u  SELECT to finish", animal_num);
		display_menu(menu_stack[menu_stack_top-1], 1);//redisplay active_menu
		
		//move curser to the digit and blink
		for (uint8_t j = 0; j<41; j++)			//go to digit screen location	
		{
			display_curser_shift(0, 1, 0);  	//shift right by one
		}

		display_onoff(1, 1, 1, 0);				//blink screen curser


		b=get_valid_pushbutton();
		
		if (VALID_UP(b))
		{
			if (animal_num<MAX_ANIMALS)
				animal_num++;
		}
		else if (VALID_DOWN(b))
		{
			if (animal_num>0)
				animal_num--;	
		}

		display_onoff(1, 0, 0, 0); 				//stop blinking screen curser
		display_home(0);

	} 
	while (!VALID_SELECT(b));

	set_animal_number(animal_num);
	active_menu.selected_line = 3;
	snprintf(active_menu.menu.screen[1], COLS+1, "%u", get_animal_number());
	display_menu(menu_stack[menu_stack_top-1], 1);//redisplay active_menu
	return 1;
}

result_t display_animal_number()
{
	san_acf_save = active_menu.custom_content_flag;
	active_menu.custom_content_flag = 0x02;
	snprintf(active_menu.menu.screen[1], COLS+1, "%u", get_animal_number());
	display_menu(menu_stack[menu_stack_top-1], 1);//redisplay active_menu
	return 1;
}

result_t destroy_animal_number()
{
	active_menu.custom_content_flag = san_acf_save;
	return 1;
}



/*
void test_menu_stack()
{
	USART_SendStack();

	push_menu(0);
	USART_SendStack();

	push_menu(1);
	USART_SendStack();

	push_menu(2);
	USART_SendStack();

	push_menu(3);
	USART_SendStack();

	push_menu(4);
	USART_SendStack();

	push_menu(5);
	USART_SendStack();

	pop_menu();
	USART_SendStack();

	push_menu(2);
	USART_SendStack();

	pop_menu();
	USART_SendStack();

	pop_menu();
	USART_SendStack();

	pop_menu();
	USART_SendStack();

	pop_menu();
	USART_SendStack();

	pop_menu();
	USART_SendStack();

	pop_menu();
	USART_SendStack();
}
*/



#endif
